;GoatTracker V2.68 song tweak utility

                processor 6502
                org 2048

SCREEN_SONG     = 0
SCREEN_PATT     = 1
SCREEN_INSTR    = 2
SCREEN_WAVE     = 3
SCREEN_PARAM    = 4

NUMINSTR        = 64
MAXMULTISPEED   = 8

        ;Zeropage variables
textlo          = $02
texthi          = $03
destlo          = $04
desthi          = $05
storea          = $06
storex          = $07
storey          = $08
trklo           = $09
trkhi           = $0a
editlo          = $0b
edithi          = $0c
freememlo       = $0d
freememhi       = $0e
temp1           = $0f
temp2           = $10
temp3           = $11
temp4           = $12
temp5           = $13
status          = $90
fa              = $ba

        ;Kernal routines
scnkey          = $ff9f
getin           = $ffe4
setnam          = $ffbd
setlfs          = $ffba
load            = $ffd5
save            = $ffd8
chrin           = $ffcf
open            = $ffc0
close           = $ffc3
chkin           = $ffc6
chkout          = $ffc9
clrchn          = $ffcc
chrout          = $ffd2
plot            = $fff0

        ;Some keycodes
KEY_F1          = $85
KEY_F3          = $86
KEY_F5          = $87
KEY_F7          = $88
KEY_UP          = $91
KEY_DOWN        = $11
KEY_LEFT        = $9d
KEY_RIGHT       = $1d
KEY_INS         = $94
KEY_DEL         = $14
KEY_BREAK       = $03
KEY_RETURN      = $0d
KEY_SPACE       = $20

start:          cld
                lda fa                  ;Find out drive number
                cmp #$08
                bcs driveok
                lda #$08
driveok:        sta drivenumber
                lda #$36                ;Switch off BASIC-rom
                sta $01
                jsr music+6             ;Make sure music is stopped
                jsr initscreen
                jsr initraster
                lda #$01                ;Print titlebar & instructions
                sta textcolor
                lda #$80
                sta bgcolor
                ldx #0
                ldy #0
                jsr setxy
                ldx #<titletext
                ldy #>titletext
                jsr printtext
                lda #$00
                sta bgcolor
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont

printall:       jsr printsong
                jsr printpatt
                jsr printinstr
                jsr printwave
                jsr printparam
branchtoscreen: lda screennum
                asl
                tax
                lda screenjumptbl,x
                sta bts_jump+1
                lda screenjumptbl+1,x
                sta bts_jump+2
bts_jump:       jmp $0000

nextscreen:     lda screennum
                clc
                adc #$01
                cmp #$05
                bcc ns_ok
                lda #$00
ns_ok:          sta screennum
                jmp printall

prevscreen:     lda screennum
                sec
                sbc #$01
                bcs ns_ok
                lda #$04
                bne ns_ok

editparam:      lda pulseopt
                and #$01
                sta pulseopt
                lda multiplier
                cmp #1
                bcs epr_multiok1
                lda #1
epr_multiok1:   cmp #MAXMULTISPEED+1
                bcc epr_multiok2
                lda #MAXMULTISPEED
epr_multiok2:   sta multiplier
                tax
                lda multitbl-1,x
                sta multiadd
                jsr printparam
                jsr commonkeys
                lda keynum
                cmp #KEY_LEFT
                bne epr_notleft
                jsr cursormoveleft
                bcc epr_back
                dec paramvx
epr_movecommon: lda paramvx
                and #$03
                sta paramvx
epr_back:       jmp editparam
epr_notleft:    cmp #KEY_RIGHT
                bne epr_notright
epr_right:      jsr cursormoveright
                bcc epr_back
epr_right2:     inc paramvx
                jmp epr_movecommon
epr_notright:   lda hexdigit
                bmi epr_nohex
                lda paramvx
                clc
                adc #<paramtbl
                sta editlo
                lda #$00
                adc #>paramtbl
                sta edithi
                jsr edithex
                bcs epr_right2
epr_nohex:      jmp editparam


editsong:       jsr printsong
                jsr commonkeys
                ldx trackvy
                lda trackadrlo,x
                clc
                adc trackvx
                sta editlo
                lda trackadrhi,x
                adc #$00
                sta edithi
                lda keynum
                cmp #KEY_RETURN
                bne es_noreturn
                ldy #$00
                lda (editlo),y
                cmp maxpatt
                bcs es_pattnotok
                sta curpatt
                lda #SCREEN_PATT
                jmp ns_ok
es_pattnotok:   jmp editsong
es_noreturn:    cmp #":"
                bne es_nosongselect1
                lda cursong
                beq editsong
                dec cursong
es_settrack:    lda cursong
                asl
                adc cursong
                sta trackvy
                sta trackmin
                clc
                adc #$02
                sta trackmax
                lda #$00
                sta trackvx
                sta startpos
                sta startpos+1
                sta startpos+2
                jmp editsong
es_nosongselect1:
                cmp #";"
                bne es_nosongselect2
                inc cursong
                lda cursong
                asl
                adc cursong
                tax
                lda tracklen,x
                bne es_settrack
                dec cursong
                jmp editsong
es_nosongselect2:
                cmp #KEY_SPACE
                bne es_nostartpos
                lda trackvy
                sbc trackmin
                tax
                lda trackvx
                sta startpos,x
                jmp editsong
es_nostartpos:  cmp #KEY_UP
                bne es_notup
                lda trackvy
                cmp trackmin
                bne es_upok
                jmp editsong
es_upok:        dec trackvy
es_checkend:    ldx trackvy
                lda trackvx
                cmp tracklen,x
                bne es_upok2
                lda #$00
                sta trackvx
es_upok2:       jmp editsong
es_notup:       cmp #KEY_DOWN
                bne es_notdown
                lda trackvy
                cmp trackmax
                bne es_downok
                jmp editsong
es_downok:      inc trackvy
                jmp es_checkend
es_notdown:     cmp #KEY_LEFT
                bne es_notleft
                jsr cursormoveleft
                bcc es_back
                lda trackvx
                bne es_leftok
                jmp editsong
es_leftok:      dec trackvx
es_back:        jmp editsong
es_notleft:     cmp #KEY_RIGHT
                bne es_notright
                jsr cursormoveright
                bcc es_back
es_right:       ldx trackvy
                lda trackvx
                clc
                adc #$01
                cmp tracklen,x
                bcc es_rightok
                jmp editsong
es_rightok:     inc trackvx
                jmp editsong
es_notright:    lda hexdigit
                bmi es_nohex
                jsr edithex
                bcc es_nohex
                jmp es_right
es_nohex:       jmp editsong

edithex:        ldy #$00
                lda cursordigit
                bne editlownybble
edithighnybble: lda hexdigit
                asl
                asl
                asl
                asl
                sta temp1
                lda (editlo),y
                and #$0f
                ora temp1
                sta (editlo),y
                inc cursordigit
                clc
                rts

editlownybble:  lda (editlo),y
                and #$f0
                ora hexdigit
                sta (editlo),y
                lda #$00
                sta cursordigit
                sec
                rts





editpatt:       jsr printpatt
                jsr commonkeys
                lda keynum
                cmp #":"
                bne ep_nopattselect1
                lda curpatt
                bne ep_ps1ok
                jmp editpatt
ep_ps1ok:       dec curpatt
ep_pscommon:    lda #$00
                sta pattvx
                sta pattvy
                jmp editpatt
ep_nopattselect1:
                cmp #";"
                bne ep_nopattselect2
                lda curpatt
                clc
                adc #$01
                cmp maxpatt
                bcc ep_ps2ok
                jmp editpatt
ep_ps2ok:       inc curpatt
                jmp ep_pscommon
ep_nopattselect2:
                cmp #KEY_UP
                bne ep_notup
                dec pattvy
                lda pattvy
                cmp #$ff
                bne ep_upok
                ldx curpatt
                lda pattlen,x
                sta pattvy
                dec pattvy
ep_upok:        jmp editpatt
ep_notup:       cmp #KEY_DOWN
                bne ep_notdown
ep_down:        inc pattvy
                ldx curpatt
                lda pattvy
                cmp pattlen,x
                bcc ep_upok
                lda #$00
                sta pattvy
                jmp editpatt
ep_notdown:     cmp #KEY_LEFT
                bne ep_notleft
                lda pattvx
                and #$01
                bne ep_leftnoadjust
                lda #$00
                sta cursordigit
ep_leftnoadjust:jsr cursormoveleft
                bcc ep_leftok
                dec pattvx
                bpl ep_leftok
                lda #$03
                sta pattvx
ep_leftok:      jmp editpatt
ep_notleft:     cmp #KEY_RIGHT
                bne ep_notright
                lda pattvx
                and #$01
                bne ep_rightnoadjust
                lda #$01
                sta cursordigit
ep_rightnoadjust:
                jsr cursormoveright
                bcc ep_leftok
                inc pattvx
                lda pattvx
                cmp #$04
                bcc ep_leftok
                lda #$00
                sta pattvx
                jmp editpatt
ep_notright:    lda pattvy
                ldx curpatt
                clc
                adc #$01
                cmp pattlen,x
                beq ep_noedit      ;No editing on end-row
                lda pattvy
                jsr multiply
                lda temp3
                clc
                adc pattvx
                sta editlo
                lda temp4
                adc #$00
                sta edithi
                lda pattvx
                beq ep_editnote
                cmp #$02
                bne ep_noeffect
                lda #$01
                sta cursordigit
ep_noeffect:    lda hexdigit
                bmi ep_nothex
                lda cursordigit
                pha
                jsr edithex
                pla
                sta cursordigit
                lda pattvx
                cmp #$01
                bne ep_noinstr
                lda (editlo),y
                and #$3f
                sta (editlo),y
ep_noinstr:     jmp ep_down
ep_noedit:
ep_nothex:      jmp editpatt

ep_editnote:    ldy #$00
                lda keynum
                cmp #$20
                bne ep_ennorest
                lda #$bd
                sta (editlo),y
                jmp ep_down
ep_ennorest:    cmp #13
                bne ep_ennokeyoff
                lda #$be
                sta (editlo),y
                jmp ep_down
ep_ennokeyoff:  cmp #$80+13
                bne ep_ennokeyon
                lda #$bf
                sta (editlo),y
                jmp ep_down
ep_ennokeyon:   cmp #"0"
                bcc ep_ennooctave
                cmp #"7"+1
                bcs ep_ennooctave
                jsr ep_getnoteoctave
                sta temp5                     ;Save note
                lda keynum
                sec
                sbc #"0"
                tax
ep_encommon:    lda octavetable,x
                clc
                adc temp5
                adc #$60
                cmp #$bd
                bcs ep_ennochange
                sta (editlo),y
                jmp ep_down
ep_ennochange:  jmp editpatt
ep_ennooctave:  ldx #11
ep_ennoteloop:  cmp notekeys,x
                beq ep_ennotefound
                dex
                bpl ep_ennoteloop
                jmp editpatt
ep_ennotefound: stx temp5                     ;Save note
                jsr ep_getnoteoctave
                jmp ep_encommon

ep_getnoteoctave:
                lda (editlo),y
                sec
                sbc #$60
                cmp #$5d
                bcs ep_noprevnote
                ldx #$ff
                sec
ep_gnoloop:     sbc #12
                inx
                bcs ep_gnoloop
ep_gnodone:     adc #12
                rts
ep_noprevnote:  lda #$00
                ldx #$00
                rts





editinstr:      jsr printinstr
                jsr commonkeys
                ldy instrvy
                lda #<instrad
                clc
                adc curinstr
                sta editlo
                lda #>instrad
                adc #$00
                sta edithi
ei_multiply:    dey
                bmi ei_multiplydone
                lda editlo
                clc
                adc #NUMINSTR
                sta editlo
                lda edithi
                adc #$00
                sta edithi
                jmp ei_multiply
ei_multiplydone:lda keynum
                cmp #KEY_RETURN
                bne ei_noreturn
                ldx instrvy
                cpx #2
                bcc ei_noreturn
                cpx #6
                bcs ei_noreturn
                ldy #$00
                lda (editlo),y
                sec
                sbc #$01
                cmp wavelen-2,x
                bcs ei_noreturn
                sta wavevy
                lda #$00
                sta wavevx
                txa
                sec
                sbc #$02
                sta tablenum
                lda #SCREEN_WAVE
                jmp ns_ok
ei_noreturn:    lda keynum
                cmp #":"
                bne ei_noselect1
                lda curinstr
                cmp #$02
                bcs ei_select1ok
                jmp editinstr
ei_select1ok:   dec curinstr
                jmp editinstr
ei_noselect1:   cmp #";"
                bne ei_noselect2
                lda curinstr
                clc
                adc #$01
                cmp maxinstr
                bcc ei_select2ok
                jmp editinstr
ei_select2ok:   inc curinstr
                jmp editinstr
ei_noselect2:   cmp #KEY_DOWN
                bne ei_notdown
ei_down:        inc instrvy
                lda instrvy
                cmp #$09
                bcc ei_downok
                lda #$00
                sta instrvy
ei_downok:      jmp editinstr
ei_notdown:     cmp #KEY_UP
                bne ei_notup
                dec instrvy
                bpl ei_upok
                lda #$08
                sta instrvy
ei_upok:        jmp editinstr
ei_notup:       cmp #KEY_LEFT
                beq ei_cursor
                cmp #KEY_RIGHT
                bne ei_nocursor
ei_cursor:      jsr cursormoveleft
                jmp editinstr
ei_nocursor:    lda hexdigit
                bmi ei_nothex
                jsr edithex
                bcc ei_nothex
                jmp ei_down
ei_nothex:      jmp editinstr





editwave:       jsr printwave
                jsr commonkeys
                ldy tablenum
                lda keynum
                cmp #":"
                bne ew_notableselect1
                dey
                cpy #$ff
                bne ew_ps1ok
                ldy #$03
ew_ps1ok:       sty tablenum
                lda #$00
                sta wavevx
                sta wavevy
                jmp editwave
ew_notableselect1:
                cmp #";"
                bne ew_notableselect2
                iny
                cpy #$04
                bne ew_ps1ok
                ldy #$00
                jmp ew_ps1ok
ew_notableselect2:
                cmp #KEY_DOWN
                bne ew_notdown
ew_down:        inc wavevy
                lda wavevy
                cmp wavelen,y
                bcc ew_downok
                lda #$00
                sta wavevy
ew_downok:      jmp editwave
ew_notdown:     cmp #KEY_UP
                bne ew_notup
                dec wavevy
                lda wavevy
                cmp #$ff
                bne ew_upok
                lda wavelen,y
                sec
                sbc #$01
                sta wavevy
ew_upok:        jmp editwave
ew_notup:       cmp #KEY_LEFT
                bne ew_notleft
                jsr cursormoveleft
                bcc ew_upok
                jmp ew_left
ew_notleft:     cmp #KEY_RIGHT
                bne ew_notright
                jsr cursormoveright
                bcc ew_upok
ew_left:        lda wavevx
                eor #$01
                sta wavevx
                jmp editwave
ew_notright:    lda hexdigit
                bmi ew_nothex
                lda tableadrlo,y
                clc
                adc wavevy
                sta editlo
                lda tableadrhi,y
                adc #$00
                adc wavevx
                sta edithi
                jsr edithex
                bcc ew_nothex
                lda wavevx
                eor #$01
                sta wavevx
                bne ew_nothex
                ldy tablenum
                jmp ew_down
ew_nothex:      jmp editwave

cursormoveleft: dec cursordigit
                bpl cml_ok
                lda #$01
                sta cursordigit
                sec
                rts
cml_ok:         clc
                rts

cursormoveright:inc cursordigit
                lda cursordigit
                cmp #$02
                bcc cmr_ok
                lda #$00
                sta cursordigit
cmr_ok:         rts

commonkeys:     jsr getkey
                sta keynum
                cmp #KEY_F3
                bne ck_noswitch
                pla
                pla
                jmp nextscreen
ck_noswitch:    cmp #KEY_F5
                bne ck_noswitch2
                pla
                pla
                jmp prevscreen
ck_noswitch2:   cmp #KEY_F1
                bne ck_noplay
ck_play:        lda playing
                eor #$01
                sta playing
                beq ck_stop
                lda cursong
                jmp music
ck_stop:        jmp music+6
ck_noplay:      cmp #"L"
                bne ck_noload
                jsr loadsong
ck_loadcommon:  pla
                pla
                jmp printall
ck_noload:      cmp #"S"
                bne ck_nosave
                jsr music+6
                jsr savesong
                jmp ck_loadcommon
ck_nosave:      rts

printsong:      lda trackvy
                sec
                sbc #$01
                sta temp1                       ;Track view Y-pos
                lda #$00                        ;Row counter
                sta temp2
pts_loop:       lda #$00
                sta bgcolor
                lda #$01
                sta textcolor
                lda temp2
                clc
                adc #$04
                tay
                ldx #$00
                jsr setxy
                lda temp1
                cmp maxtracks
                bcc pts_notempty
                jmp pts_empty
pts_notempty:   ldx temp1
                lda trknumtbl1,x
                sta pts_text+3
                lda trknumtbl2,x
                sta pts_text+4
                lda trknumtbl3,x
                sta pts_text+9
                lda trackadrlo,x
                sta temp4
                lda trackadrhi,x
                sta temp5
                lda #$01
                sta textcolor
                ldx #<pts_text
                ldy #>pts_text
                jsr printtext
                lda #11
                sta cursorx
                lda trackvx
                sec
                sbc #$04
                sta temp3               ;Track view X-pos
pts_xloop:      lda #$00
                sta bgcolor
                lda #$0f
                sta textcolor
                lda trackvx
                cmp temp3
                bne pts_nocursor
                lda trackvy
                cmp temp1
                bne pts_nocursor
                lda screennum
                cmp #SCREEN_SONG
                bne pts_nocursor
                lda #$c0
                sta bgcolor
pts_nocursor:   ldx temp1
                lda temp3
                cmp tracklen,x
                bcs pts_xempty
                tay
                lda temp1
                sec
                sbc trackmin
                cmp #$03
                bcs pts_nostart
                tax
                tya
                cmp startpos,x
                bne pts_nostart
                lda #$01
                sta textcolor
pts_nostart:    lda (temp4),y
                jsr printhex8
                jmp pts_xdone
pts_xempty:     ldx #<pts_emptytxt2
                ldy #>pts_emptytxt2
                jsr printtext
pts_xdone:      inc temp3
                lda cursorx
                clc
                adc #$03
                sta cursorx
                lda cursorx
                cmp #39
                bcc pts_xloop
                jmp pts_done
pts_empty:      jsr printemptyrow
pts_done:       inc temp1
                inc temp2
                lda temp2
                cmp #$03
                bcs pts_alldone
                jmp pts_loop
pts_alldone:    rts

printpatt:      lda #$00
                sta bgcolor
                ldx #0
                ldy #8
                jsr setxy
                lda #$01
                sta textcolor
                ldx #<ptp_text
                ldy #>ptp_text
                jsr printtext
                lda #7
                sta cursorx
                lda curpatt
                jsr printhex8
                lda pattvy
                sec
                sbc #$07
                sta temp1
                lda #10
                sta temp2
ptp_loop:       ldx #$00
                stx bgcolor
                ldy temp2
                jsr setxy
                ldx curpatt
                lda temp1
                cmp pattlen,x
                bcs ptp_empty
                jsr multiply
                lda #$01
                sta textcolor
                lda temp1
                jsr printhex8
                lda #$0f
                sta textcolor
                lda #$03
                sta cursorx

                ldy #0             ;Note
                jsr pattcursor
                lda #$00
                sta temp5
                lda (temp3),y
                cmp #$ff
                beq ptp_endrow
                sec
                sbc #$60
                cmp #$60
                bcc noteok
                lda #$5d
noteok:         asl
                rol temp5
                asl
                rol temp5
                clc
                adc #<ptp_notename
                tax
                lda temp5
                adc #>ptp_notename
                tay
                jsr printtext
                lda #$06
                sta cursorx

                ldy #1
                jsr pattcursor      ;Instrument
                lda (temp3),y
                jsr printhex8
                lda #$08
                sta cursorx

                ldy #2
                jsr pattcursor
                lda (temp3),y       ;Effect
                jsr printhex4
                inc cursorx

                ldy #3
                jsr pattcursor
                lda (temp3),y       ;Parameter
                jsr printhex8
                jmp ptp_done

ptp_endrow:     ldx #<ptp_endtxt
                ldy #>ptp_endtxt
                bne ptp_common
ptp_empty:      ldx #<ptp_emptytxt
                ldy #>ptp_emptytxt
ptp_common:     jsr printtext
ptp_done:       inc temp1
                inc temp2
                lda temp2
                cmp #24
                bcs ptp_alldone
                jmp ptp_loop
ptp_alldone:    rts

pattcursor:     cpy pattvx
                bne pattcursoroff
                lda temp1
                cmp pattvy
                bne pattcursoroff
                lda screennum
                cmp #SCREEN_PATT
                bne pattcursoroff
                lda #$c0
                sta bgcolor
                rts
pattcursoroff:  lda #$00
                sta bgcolor
                rts

multiply:       sta temp3
                lda #$00
                sta temp4
                asl temp3
                rol temp4
                asl temp3
                rol temp4
                lda pattadrlo,x
                clc
                adc temp3
                sta temp3
                lda pattadrhi,x
                adc temp4
                sta temp4
                rts

printinstr:     lda #$00
                sta bgcolor
                ldx #13
                ldy #8
                jsr setxy
                lda #$01
                sta textcolor
                ldx #<pti_text
                ldy #>pti_text
                jsr printtext
                inc cursory
                inc cursory
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                inc cursory
                jsr printtextcont
                ldx #23
                ldy #8
                jsr setxy
                lda curinstr
                jsr printhex8
                lda #$0f
                sta textcolor
                inc cursory
                inc cursory
                inc cursory
                lda curinstr
                sta printname+1
                lda #$00
                sta printname+2
                ldy #$04
printnamemul:   asl printname+1
                rol printname+2
                dey
                bne printnamemul
                lda printname+1
                adc #<instrnames
                sta printname+1
                lda printname+2
                adc #>instrnames
                sta printname+2
                ldx #$0f
printname:      lda instrnames,x
                bne printname_nozero
                lda #$20
printname_nozero:
                cmp #96
                bcc printname_uppercase
                sbc #96
printname_uppercase:
                and #$3f
                sta $0400+9*40+13,x
                lda #$0f
                sta $d800+9*40+13,x
                dex
                bpl printname
                ldx curinstr
                ldy #$00
                jsr instrcursor
                lda instrad,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$01
                jsr instrcursor
                lda instrsr,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$02
                jsr instrcursor
                lda instrwavepos,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$03
                jsr instrcursor
                lda instrpulsepos,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$04
                jsr instrcursor
                lda instrfiltpos,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$05
                jsr instrcursor
                lda instrvibpos,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$06
                jsr instrcursor
                lda instrvibdelay,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$07
                jsr instrcursor
                lda instrgatetimer,x
                jsr printhex8
                inc cursory
                ldx curinstr
                ldy #$08
                jsr instrcursor
                lda instr1stwave,x
                jsr printhex8
                rts

instrcursor:    lda screennum
                cmp #SCREEN_INSTR
                bne instrcursoroff
                cpy instrvy
                bne instrcursoroff
                lda #$c0
                sta bgcolor
                rts
instrcursoroff: lda #$00
                sta bgcolor
                rts


printparam:     lda #$00
                sta bgcolor
                ldx #13
                ldy #21
                jsr setxy
                lda #$01
                sta textcolor
                ldx #<ptpr_text
                ldy #>ptpr_text
                jsr printtext
                lda #$0f
                sta textcolor
                lda #13
                sta temp1
                lda #0
                sta temp2
ppr_loop:       ldx temp1
                ldy #23
                jsr setxy
                ldx temp2
                jsr paramcursor
                lda paramtbl,x
                jsr printhex8
                lda temp1
                clc
                adc #$03
                sta temp1
                inc temp2
                lda temp2
                cmp #$04
                bcc ppr_loop
                rts

paramcursor:    lda screennum
                cmp #SCREEN_PARAM
                bne paramcursoroff
                cpx paramvx
                bne paramcursoroff
                lda #$c0
                sta bgcolor
                rts
paramcursoroff: lda #$00
                sta bgcolor
                rts

printwave:      lda #$00
                sta bgcolor
                ldx #29
                ldy #8
                jsr setxy
                lda #$01
                sta textcolor
                ldx tablenum
                lda tableadrlo,x
                sta ptw_lda1+1
                sta ptw_lda2+1
                lda tableadrhi,x
                sta ptw_lda1+2
                clc
                adc #$01
                sta ptw_lda2+2
                lda ptw_texttbllo,x
                ldy ptw_texttblhi,x
                tax
                jsr printtext
                lda wavevy
                sec
                sbc #$07
                sta temp1
                lda #10
                sta temp2
ptw_loop:       lda #$00
                sta bgcolor
                ldy temp2
                ldx #29
                jsr setxy
                ldx tablenum
                lda temp1
                cmp wavelen,x
                bcs ptw_empty
                lda #$01
                sta textcolor
                lda #$00
                sta bgcolor
                lda temp1
                clc
                adc #$01
                jsr printhex8
                lda #$0f
                sta textcolor
                ldx #32
                stx cursorx
                ldx #$00
                jsr wavecursor
                ldy temp1
ptw_lda1:       lda wavetbl,y
                jsr printhex8
                ldx #35
                stx cursorx
                ldx #$01
                jsr wavecursor
                ldy temp1
ptw_lda2:       lda notetbl,y
                jsr printhex8
                jmp ptw_done
ptw_empty:      ldx #<ptp_emptytxt
                ldy #>ptp_emptytxt
                jsr printtext
ptw_done:       inc temp1
                inc temp2
                lda temp2
                cmp #24
                bcc ptw_loop
                rts

wavecursor:     lda screennum
                cmp #SCREEN_WAVE
                bne wavecursoroff
                lda temp1
                cmp wavevy
                bne wavecursoroff
                cpx wavevx
                bne wavecursoroff
                lda #$c0
                sta bgcolor
                rts
wavecursoroff:  lda #$00
                sta bgcolor
                rts

        ;Gethex - converts keypress to hex
gethex:         cmp #"0"
                bcc gh_hnotnum
                cmp #"9"+1
                bcs gh_hnotnum
                sbc #"0"-1
                jmp gh_h2
gh_hnotalpha:   lda #$ff
                sta hexdigit
                rts
gh_hnotnum:     cmp #"A"
                bcc gh_hnotalpha
                cmp #"F"+1
                bcs gh_hnotalpha
                sbc #"A"-11
gh_h2:          sta hexdigit
                rts


loadsong:       jsr askname
                lda namelength
                bne loadok
                rts
loadok:         jsr music+6
                lda #$00
                sta playing
                sta cursong
                sta curpatt
                sta trackmin
                lda #$02
                sta trackmax
                lda #$01
                sta curinstr
                lda #$00
                sta instrvy
                sta wavevy
                sta pattvy
                sta trackvy
                sta pattvx
                sta instrvx
                sta trackvx
                sta startpos
                sta startpos+1
                sta startpos+2
                lda #<freemem
                sta freememlo
                lda #>freemem
                sta freememhi
                lda #$02
                ldy #$00
                ldx drivenumber
                jsr setlfs
                clc
                lda namelength
                ldx #<name
                ldy #>name
                jsr setnam
                jsr open
                ldx #$02
                jsr chkin
                ldy #$00

loadident:      jsr chrin                       ;Read identification (4 bytes)
                ldx status
                beq load_identok
                jmp load_error
load_identok:   sta ident,y
                iny
                cpy #$04
                bcc loadident
                lda ident+3
                cmp #"5"                        ;Check for V2.59 operation
                beq load_identok2
                jmp load_error

load_identok2:  ldy #$00
loadinfo:       jsr load_getbyte                ;Read authorinfo (96 bytes)
                sta info,y
                iny
                cpy #96
                bcc loadinfo

                jsr load_getbyte
                sta maxsongs                    ;Store amount of songs
                asl
                adc maxsongs
                sta maxtracks                   ;Store amount of tracks
                sta temp1                       ;Track counter
                lda #$00
                sta temp2
loadtracks:     ldy temp2
                lda freememlo
                sta trackadrlo,y
                lda freememhi
                sta trackadrhi,y
                jsr load_getbyte                ;Get track length
                clc
                adc #$01                        ;Increment by one
                sta tracklen,y
                sta temp3
loadtrackloop:  jsr load_getbyte
                ldy #$00
                sta (freememlo),y
                jsr incfreemem
                dec temp3
                bne loadtrackloop
                inc temp2
                dec temp1
                bne loadtracks                  ;Loop until all tracks read

                lda #$01
                sta temp1
                jsr load_getbyte                ;Get amount of instruments
                clc
                adc #$01
                sta maxinstr
                lda #<(instrnames+$10)
                sta editlo
                lda #>(instrnames+$10)
                sta edithi
loadinstrloop:  ldy temp1
                jsr load_getbyte
                sta instrad,y
                jsr load_getbyte
                sta instrsr,y
                jsr load_getbyte
                sta instrwavepos,y
                jsr load_getbyte
                sta instrpulsepos,y
                jsr load_getbyte
                sta instrfiltpos,y
                jsr load_getbyte
                sta instrvibpos,y
                jsr load_getbyte
                sta instrvibdelay,y
                jsr load_getbyte
                sta instrgatetimer,y
                jsr load_getbyte
                sta instr1stwave,y
                lda #$10
                sta temp2
loadinstrname:  jsr load_getbyte                ;Load the 16-byte instrument
                ldy #$00                        ;name
                sta (editlo),y
                inc editlo
                bne lir_ok
                inc edithi
lir_ok:         dec temp2
                bne loadinstrname
                inc temp1
                lda temp1
                cmp maxinstr
                bcc loadinstrloop

                jsr load_getbyte                ;Get length of wave table
                sta wavelen
                ldy #$00
                cpy wavelen
                beq loadskipwave
loadwavetable:  jsr load_getbyte                ;Load table left side
                sta wavetbl,y
                iny
                cpy wavelen
                bne loadwavetable
                ldy #$00
loadnotetable:  jsr load_getbyte                ;Load table right side
                sta notetbl,y
                iny
                cpy wavelen
                bne loadnotetable

loadskipwave:   jsr load_getbyte                ;Get length of pulse table
                sta pulselen
                ldy #$00
                cpy pulselen
                beq loadskippulse
loadpulsetimetable:
                jsr load_getbyte                ;Load table left side
                sta pulsetimetbl,y
                iny
                cpy pulselen
                bne loadpulsetimetable
                ldy #$00
loadpulsespdtable:
                jsr load_getbyte                ;Load table right side
                sta pulsespdtbl,y
                iny
                cpy pulselen
                bne loadpulsespdtable

loadskippulse:  jsr load_getbyte                ;Get length of filter table
                sta filtlen
                ldy #$00
                cpy filtlen
                beq loadskipfilter
loadfilttimetable:
                jsr load_getbyte                ;Load table left side
                sta filttimetbl,y
                iny
                cpy filtlen
                bne loadfilttimetable
                ldy #$00
loadfiltspdtable:
                jsr load_getbyte                ;Load table right side
                sta filtspdtbl,y
                iny
                cpy filtlen
                bne loadfiltspdtable

loadskipfilter: jsr load_getbyte                ;Get length of speed table
                sta speedlen
                ldy #$00
                cpy speedlen
                beq loadskipspeed
loadspeedlefttable:
                jsr load_getbyte                ;Load table left side
                sta speedlefttbl,y
                iny
                cpy speedlen
                bne loadspeedlefttable
                ldy #$00
loadspeedrighttable:
                jsr load_getbyte                ;Load table right side
                sta speedrighttbl,y
                iny
                cpy speedlen
                bne loadspeedrighttable

loadskipspeed:  jsr load_getbyte                ;Amount of patterns
                sta maxpatt
                lda #$00
                sta temp2
loadpattloop:   ldy temp2
                lda #$00
                sta temp3
                jsr load_getbyte                ;Length of pattern
                sta pattlen,y
                sta temp4
                lda freememlo
                sta pattadrlo,y
                lda freememhi
                sta pattadrhi,y
loadpattrowloop:ldy #$00
                jsr load_getbyte
                sta (freememlo),y
                jsr incfreemem
                jsr load_getbyte
                sta (freememlo),y
                jsr incfreemem
                jsr load_getbyte
                sta (freememlo),y
                jsr incfreemem
                jsr load_getbyte
                sta (freememlo),y
                jsr incfreemem
                inc temp3
                lda temp3
                cmp temp4
                bcc loadpattrowloop
                inc temp2
                lda temp2
                cmp maxpatt
                bcc loadpattloop

                jmp load_error          ;All done!

load_getbyte:   ldx status
                bne load_fileend
                jsr chrin
                dec $d020
                inc $d020
                rts
load_fileend:   pla
                pla
load_error:     lda #$02
                jsr close
                jsr clrchn
                rts

incfreemem:     inc freememlo
                bne ifm_done
                inc freememhi
ifm_done:       rts

savesong:       jsr askname
                lda namelength
                bne saveok
                rts
saveok:         jsr music+6
                lda #$00
                sta playing
                lda #15
                ldx drivenumber
                ldy #15
                jsr setlfs
                clc
                lda namelength
                adc #$03
                ldx #<scratch
                ldy #>scratch
                jsr setnam
                jsr open
                lda #15
                jsr close
                lda #$02
                ldy #$01
                ldx drivenumber
                jsr setlfs
                lda namelength
                ldx #<name
                ldy #>name
                jsr setnam
                lda namelength
                jsr open
                ldx #$02
                jsr chkout
                ldy #$00
saveident:      lda ident,y
                jsr save_sendbyte                ;Write identification (4 bytes)
                iny
                cpy #$04
                bcc saveident
                ldy #$00
saveinfo:       lda info,y
                jsr save_sendbyte                ;Write authorinfo (96 bytes)
                iny
                cpy #96
                bcc saveinfo
                lda maxsongs                     ;Store amount of songs
                jsr save_sendbyte
                lda maxtracks
                sta temp1                        ;Track counter
                lda #$00
                sta temp2
savetracks:     ldy temp2
                lda trackadrlo,y
                sta freememlo
                lda trackadrhi,y
                sta freememhi
                lda tracklen,y
                sta temp3
                sec
                sbc #$01
                jsr save_sendbyte                ;Store track length
savetrackloop:  ldy #$00
                lda (freememlo),y
                jsr save_sendbyte
                jsr incfreemem
                dec temp3
                bne savetrackloop
                inc temp2
                dec temp1
                bne savetracks                  ;Loop until all tracks written
                lda #$01
                sta temp1
                lda #<(instrnames+$10)
                sta editlo
                lda #>(instrnames+$10)
                sta edithi
                lda maxinstr
                sec
                sbc #$01                        ;Save amount of instruments
                jsr save_sendbyte
saveinstrloop:  ldy temp1
                lda instrad,y
                jsr save_sendbyte
                lda instrsr,y
                jsr save_sendbyte
                lda instrwavepos,y
                jsr save_sendbyte
                lda instrpulsepos,y
                jsr save_sendbyte
                lda instrfiltpos,y
                jsr save_sendbyte
                lda instrvibpos,y
                jsr save_sendbyte
                lda instrvibdelay,y
                jsr save_sendbyte
                lda instrgatetimer,y
                jsr save_sendbyte
                lda instr1stwave,y
                jsr save_sendbyte
                lda #$10
                sta temp2
saveinstrname:  ldy #$00                        ;Store instr.name
                lda (editlo),y
                jsr save_sendbyte
                inc editlo
                bne sir_ok
                inc edithi
sir_ok:         dec temp2
                bne saveinstrname
                inc temp1
                lda temp1
                cmp maxinstr                    ;Loop until all instruments written?
                bcc saveinstrloop

                lda wavelen
                jsr save_sendbyte                ;Get length of wave table
                ldy #$00
                cpy wavelen
                beq savewaveskip
savewavetable:  lda wavetbl,y
                jsr save_sendbyte                ;Save table left side
                iny
                cpy wavelen
                bne savewavetable
                ldy #$00
savenotetable:  lda notetbl,y
                jsr save_sendbyte                ;Save table right side
                iny
                cpy wavelen
                bne savenotetable

savewaveskip:   lda pulselen
                jsr save_sendbyte                ;Get length of pulse table
                ldy #$00
                cpy pulselen
                beq savepulseskip
savepulsetimetable:
                lda pulsetimetbl,y
                jsr save_sendbyte                ;Save table left side
                iny
                cpy pulselen
                bne savepulsetimetable
                ldy #$00
savepulsespdtable:
                lda pulsespdtbl,y
                jsr save_sendbyte                ;Save table right side
                iny
                cpy pulselen
                bne savepulsespdtable

savepulseskip:  lda filtlen
                jsr save_sendbyte                ;Get length of filter table
                ldy #$00
                cpy filtlen
                beq savefiltskip
savefilttimetable:
                lda filttimetbl,y
                jsr save_sendbyte                ;save table left side
                iny
                cpy filtlen
                bne savefilttimetable
                ldy #$00
savefiltspdtable:
                lda filtspdtbl,y
                jsr save_sendbyte                ;save table right side
                iny
                cpy filtlen
                bne savefiltspdtable

savefiltskip:   lda speedlen
                jsr save_sendbyte                ;Get length of speed table
                ldy #$00
                cpy speedlen
                beq savespeedskip
savespeedlefttable:
                lda speedlefttbl,y
                jsr save_sendbyte                ;save table left side
                iny
                cpy speedlen
                bne savespeedlefttable
                ldy #$00
savespeedrighttable:
                lda speedrighttbl,y
                jsr save_sendbyte                ;save table right side
                iny
                cpy speedlen
                bne savespeedrighttable

savespeedskip:  lda maxpatt
                sta temp1
                jsr save_sendbyte                ;Amount of patterns
                lda #$00
                sta temp2
savepattloop:   ldy temp2
                lda pattlen,y
                sta temp3
                jsr save_sendbyte                ;Length of pattern
                lda pattadrlo,y
                sta freememlo
                lda pattadrhi,y
                sta freememhi
savepattrowloop:ldy #$00
                lda (freememlo),y
                jsr save_sendbyte
                jsr incfreemem
                lda (freememlo),y
                jsr save_sendbyte
                jsr incfreemem
                lda (freememlo),y
                jsr save_sendbyte
                jsr incfreemem
                lda (freememlo),y
                jsr save_sendbyte
                jsr incfreemem
                dec temp3
                bne savepattrowloop
                inc temp2
                dec temp1
                bne savepattloop
                jmp load_error          ;All done!

save_sendbyte:  jsr chrout
                ldx status
                bne save_senderror
                dec $d020
                inc $d020
                rts
save_senderror: pla
                pla
save_done:      jmp load_error

askname:        lda #$00
                sta namelength
                ldx #$0f
                lda #$20
askname1:       sta name,x
                dex
                bpl askname1
                ldx #0
                ldy #24
                jsr setxy
                ldx #<asknametext
                ldy #>asknametext
                jsr printtext
asknameloop:    ldx #9
                ldy #24
                jsr setxy
                ldx #<name
                ldy #>name
                jsr printtext
                lda #$20
                sta $400+24*40+16+9
                ldx namelength
                lda $400+24*40+9,x
                ora #$c0
                sta $400+24*40+9,x
                jsr getkey
                cmp #KEY_RETURN
                beq askname_end
                cmp #KEY_DEL
                bne askname_nodel
                lda namelength
                beq asknameloop
                dec namelength
                ldx namelength
                lda #$20
                sta name,x
                jmp asknameloop
askname_nodel:  cmp #$20
                bcc asknameloop
                and #$3f
                ldx namelength
                cpx #$10
                bcs asknameloop
                sta name,x
                inc namelength
                jmp asknameloop
askname_end:    ldx #$0f
askname_final:  lda name,x
                cmp #$20
                bcs askname_final2
                ora #$40
                sta name,x
askname_final2: dex
                bpl askname_final
                lda #24
                sta cursory
                jmp printemptyrow

getkey:         jsr getkey_delay
                jsr scnkey
                jsr getin
                cmp #$00
                bne getkey_done
                jmp getkey
getkey_done:    pha
                jsr gethex
                pla
                rts

getkey_delay:   ldy #6
gkd1:           ldx #$ff
gkd2:           dex
                bne gkd2
                dey
                bne gkd1
                rts

waitkey:        jsr getkey_delay
                jsr getkey_delay
                jsr scnkey
                jsr getin
                cmp #$00
                beq waitkey
                rts

initscreen:     lda #$00
                sta $d020
                sta $d021
                lda #$0b
                sta $d022
                lda #$0c
                sta $d023
                lda #$03
                sta $dd00
                lda #$08
                sta $d016
                lda #$5b                ;Extended colormode
                sta $d011
                lda #21
                sta $d018
                lda #$ff
                sta 650
clearscreen:    lda #$20
                ldx #$00
is_loop:        sta $0400,x
                sta $0500,x
                sta $0600,x
                sta $0700,x
                inx
                bne is_loop
                rts

        ;X=xposition, Y=yposition
setxy:          stx cursorx
                sty cursory
                rts

printemptyrow:  ldx #$00
                stx cursorx
                ldx #<emptyrow
                ldy #>emptyrow

        ;X=text address lo, Y=text address hi
printtext:      stx textlo
                sty texthi
printtextcont:  ldx cursory
                clc
                lda rowtbllo,x
                adc cursorx
                sta destlo
                lda rowtblhi,x
                adc #$00
                sta desthi
                ldy #$00
pt_loop:        lda (textlo),y
                beq pt_done
                and #$3f
                ora bgcolor
                sta (destlo),y
                clc
                lda desthi
                adc #$d4
                sta desthi
                lda textcolor
                sta (destlo),y
                sec
                lda desthi
                sbc #$d4
                sta desthi
                iny
                jmp pt_loop
pt_done:        iny
                tya
                clc
                adc textlo
                sta textlo
                lda texthi
                adc #$00
                sta texthi
                rts

        ;A=hex value
printhex4:      jsr converthex4
                ldx cursory
                clc
                lda rowtbllo,x
                adc cursorx
                sta destlo
                lda rowtblhi,x
                adc #$d4
                sta desthi
                ldy #$00
                lda textcolor
                sta (destlo),y
                lda desthi
                sec
                sbc #$d4
                sta desthi
                lda hexstring+1
                and #$3f
                ora bgcolor
                sta (destlo),y
                rts

        ;A=hex value
printhex8:      jsr converthex8
                ldx cursory
                clc
                lda rowtbllo,x
                adc cursorx
                sta destlo
                lda rowtblhi,x
                adc #$d4
                sta desthi
                lda textcolor
                ldy #$00
                sta (destlo),y
                iny
                sta (destlo),y
                dey
                lda desthi
                sec
                sbc #$d4
                sta desthi
                lda cursordigit
                beq printhex_left
                lda hexstring
                and #$3f
                ldy #$00
                sta (destlo),y
                lda hexstring+1
                and #$3f
                ora bgcolor
                iny
                sta (destlo),y
                rts
printhex_left:  lda hexstring
                and #$3f
                ora bgcolor
                ldy #$00
                sta (destlo),y
                lda hexstring+1
                and #$3f
                iny
                sta (destlo),y
                rts

converthex8:    sta hexvalue
                lsr
                lsr
                lsr
                lsr
                tax
                lda hexcodes,x
                sta hexstring
                lda hexvalue
converthex4:    and #$0f
                tax
                lda hexcodes,x
                sta hexstring+1
                rts

initraster:     sei
                lda #$7f
                sta $dc0d
                lda #$01
                sta $d01a
                lda #<raster
                sta $0314
                lda #>raster
                sta $0315
                lda lastraster
                sta $d012
                lda $dc0d
                cli
                rts

raster:         inc $d019
                jsr music+3
                lda lastraster
                bne ras_nocursor
                lda cursorcolor
                and #$1f
                tax
                lda cursorcolortbl,x
                sta $d024
                inc cursorcolor
ras_nocursor:   lda multiadd
                beq ras_nomulti
                clc
                adc lastraster
                bcc ras_nomulti
                lda #$00
ras_nomulti:    sta lastraster
                sta $d012
                jmp $ea81

screenjumptbl:  dc.w editsong
                dc.w editpatt
                dc.w editinstr
                dc.w editwave
                dc.w editparam

ptw_texttbllo:  dc.b <ptw_text1
                dc.b <ptw_text2
                dc.b <ptw_text3
                dc.b <ptw_text4
ptw_texttblhi:  dc.b >ptw_text1
                dc.b >ptw_text2
                dc.b >ptw_text3
                dc.b >ptw_text4
tableadrlo:     dc.b <wavetbl
                dc.b <pulsetimetbl
                dc.b <filttimetbl
                dc.b <speedlefttbl
tableadrhi:     dc.b >wavetbl
                dc.b >pulsetimetbl
                dc.b >filttimetbl
                dc.b >speedlefttbl

trknumtbl1:     repeat 30
                dc.b "0"
                repend
                repeat 30
                dc.b "1"
                repend
                repeat 30
                dc.b "2"
                repend
                repeat 6
                dc.b "3"
                repend

trknumtbl2:     repeat 3
                dc.b "000111222333444555666777888999"
                repend
                dc.b "000111"

trknumtbl3:     repeat 32
                dc.b "123"
                repend

cursorcolor:    dc.b 0
cursorcolortbl: dc.b 0,0,9,9,2,2,8,8
                dc.b 10,10,15,15,7,7,1,1
                dc.b 1,1,7,7,15,15,10,10
                dc.b 8,8,2,2,9,9,0,0

hexvalue:       dc.b 0
hexvalue2:      dc.b 0

hexstring:      dc.b "00",0
hexcodes:       dc.b "0123456789ABCDEF",0

scratch:        dc.b "S0:"
name:           dc.b "                ",0
namelength:     dc.b 0
drivenumber:    dc.b 8

cursorx:        dc.b 0
cursory:        dc.b 0
bgcolor:        dc.b 0
textcolor:      dc.b 1
keynum:         dc.b 0
hexdigit:       dc.b 0
playing:        dc.b 0

startpos:       dc.b 0,0,0

rowtbllo:
N               SET 0
                REPEAT 25
                dc.b <($400+40*N)
N               SET N+1
                REPEND

rowtblhi:
N               SET 0
                REPEAT 25
                dc.b >($400+40*N)
N               SET N+1
                REPEND

pattadrlo:      ds.b 256,<freemem
pattadrhi:      ds.b 256,>freemem

trackadrlo:     ds.b 32*3,<freemem
trackadrhi:     ds.b 32*3,>freemem

pattlen:        dc.b 1
                ds.b 255,0
tracklen:       dc.b 1,1,1
                ds.b 31*3,0

trackvx:        dc.b 0
trackvy:        dc.b 0
trackmin:       dc.b 0
trackmax:       dc.b 2

pattvx:         dc.b 0
pattvy:         dc.b 0

instrvx:        dc.b 0
instrvy:        dc.b 0

wavevx:         dc.b 0
wavevy:         dc.b 0
tablenum:       dc.b 0

paramvx:        dc.b 0

maxpatt:        dc.b 1
maxsongs:       dc.b 1
maxtracks:      dc.b 3
maxinstr:       dc.b NUMINSTR

screennum:      dc.b SCREEN_SONG
curpatt:        dc.b 0
cursong:        dc.b 0
curinstr:       dc.b 1
cursordigit:    dc.b 0

instrad:        ds.b NUMINSTR,0
instrsr:        ds.b NUMINSTR,0
instrwavepos:   ds.b NUMINSTR,0
instrpulsepos:  ds.b NUMINSTR,0
instrfiltpos:   ds.b NUMINSTR,0
instrvibpos:    ds.b NUMINSTR,0
instrvibdelay:  ds.b NUMINSTR,0
instrgatetimer: ds.b NUMINSTR,0
instr1stwave:   ds.b NUMINSTR,0

wavelen:        dc.b 1
pulselen:       dc.b 1
filtlen:        dc.b 1
speedlen:       dc.b 1

wavetbl:        ds.b 256,0
notetbl:        ds.b 256,0

pulsetimetbl:   ds.b 256,0
pulsespdtbl:    ds.b 256,0

filttimetbl:    ds.b 256,0
filtspdtbl:     ds.b 256,0

                dc.b 0
speedlefttbl:   ds.b 256,0
                dc.b 0
speedrighttbl:  ds.b 256,0

paramtbl:
pulseopt:       dc.b 1
multiplier:     dc.b 1
adparam:        dc.b $0f
srparam:        dc.b $00

multiadd:       dc.b $00
lastraster:     dc.b $00

multitbl:       dc.b 0
N               set 2
                repeat MAXMULTISPEED-1
                dc.b 290/N
N               set N+1
                repend

instrnames:     ds.b 16*NUMINSTR,$20

ident:          ds.b 4,0
info:           ds.b 96,0

notekeys:       dc.b "C"
                dc.b "C"+$80
                dc.b "D"
                dc.b "D"+$80
                dc.b "E"
                dc.b "F"
                dc.b "F"+$80
                dc.b "G"
                dc.b "G"+$80
                dc.b "A"
                dc.b "A"+$80
                dc.b "B"

octavetable:    dc.b 0,12,24,36,48,60,72,84

emptyrow:       dc.b "                                        ",0
titletext:      dc.b "  GOATTRACKER V2.68 SONG TWEAK UTILITY  ",0
helptext:       dc.b " L=LOAD S=SAVE F1=START/STOP F3/F5=MODE ",0
helptext2:      dc.b " SPC=STARTPOS :;=SEL.SONG/PATT/INS/TBL. ",0
asknametext:    dc.b "FILENAME:",0

pts_text:       dc.b "SNG00 TRK0",0
pts_emptytxt2:  dc.b "  ",0

ptp_text:       dc.b "PATTERN",0
ptp_emptytxt:   dc.b "           ",0
ptp_endtxt:     dc.b "END PATT",0

ptp_notename:   dc.b "C-0",0
                dc.b "C#0",0
                dc.b "D-0",0
                dc.b "D#0",0
                dc.b "E-0",0
                dc.b "F-0",0
                dc.b "F#0",0
                dc.b "G-0",0
                dc.b "G#0",0
                dc.b "A-0",0
                dc.b "A#0",0
                dc.b "B-0",0
                dc.b "C-1",0
                dc.b "C#1",0
                dc.b "D-1",0
                dc.b "D#1",0
                dc.b "E-1",0
                dc.b "F-1",0
                dc.b "F#1",0
                dc.b "G-1",0
                dc.b "G#1",0
                dc.b "A-1",0
                dc.b "A#1",0
                dc.b "B-1",0
                dc.b "C-2",0
                dc.b "C#2",0
                dc.b "D-2",0
                dc.b "D#2",0
                dc.b "E-2",0
                dc.b "F-2",0
                dc.b "F#2",0
                dc.b "G-2",0
                dc.b "G#2",0
                dc.b "A-2",0
                dc.b "A#2",0
                dc.b "B-2",0
                dc.b "C-3",0
                dc.b "C#3",0
                dc.b "D-3",0
                dc.b "D#3",0
                dc.b "E-3",0
                dc.b "F-3",0
                dc.b "F#3",0
                dc.b "G-3",0
                dc.b "G#3",0
                dc.b "A-3",0
                dc.b "A#3",0
                dc.b "B-3",0
                dc.b "C-4",0
                dc.b "C#4",0
                dc.b "D-4",0
                dc.b "D#4",0
                dc.b "E-4",0
                dc.b "F-4",0
                dc.b "F#4",0
                dc.b "G-4",0
                dc.b "G#4",0
                dc.b "A-4",0
                dc.b "A#4",0
                dc.b "B-4",0
                dc.b "C-5",0
                dc.b "C#5",0
                dc.b "D-5",0
                dc.b "D#5",0
                dc.b "E-5",0
                dc.b "F-5",0
                dc.b "F#5",0
                dc.b "G-5",0
                dc.b "G#5",0
                dc.b "A-5",0
                dc.b "A#5",0
                dc.b "B-5",0
                dc.b "C-6",0
                dc.b "C#6",0
                dc.b "D-6",0
                dc.b "D#6",0
                dc.b "E-6",0
                dc.b "F-6",0
                dc.b "F#6",0
                dc.b "G-6",0
                dc.b "G#6",0
                dc.b "A-6",0
                dc.b "A#6",0
                dc.b "B-6",0
                dc.b "C-7",0
                dc.b "C#7",0
                dc.b "D-7",0
                dc.b "D#7",0
                dc.b "E-7",0
                dc.b "F-7",0
                dc.b "F#7",0
                dc.b "G-7",0
                dc.b "G#7",0
                dc.b "...",0
                dc.b "---",0
                dc.b "+++",0

pti_text:       dc.b "INSTRUMENT",0
pti_text2:      dc.b "ATK/DEC",0
pti_text3:      dc.b "SUS/REL",0
pti_text4:      dc.b "WAVE POS",0
pti_text5:      dc.b "PULSE POS",0
pti_text6:      dc.b "FILT POS",0
pti_text7:      dc.b "VIB POS",0
pti_text8:      dc.b "VIB DELAY",0
pti_text9:      dc.b "GATETIMER",0
pti_text10:     dc.b "1ST WAVE",0

ptpr_text:      dc.b "PARAMETERS",0
ptw_text1:      dc.b "WAVETABLE",0
ptw_text2:      dc.b "PULSTABLE",0
ptw_text3:      dc.b "FILTTABLE",0
ptw_text4:      dc.b "SPD-TABLE",0




;-------------------------------------------------------------------------------
; GoatTracker V2.2 standard playroutine with unbuffered writes.
;
; NOTE: This playroutine source code does not fall under the GPL license!
; Use it freely for any purpose, commercial or noncommercial.
;-------------------------------------------------------------------------------

        ; Used zeropage temp variables

mt_temp1        = $fc
mt_temp2        = $fd

        ; Defines for the music data
        ; Patterndata notes

ENDPATT         = $ff
NOTE            = $60
REST            = $bd
KEYOFF          = $be
KEYON           = $bf

        ;Effects

DONOTHING    = $00
PORTAUP      = $01
PORTADOWN    = $02
TONEPORTA    = $03
VIBRATO      = $04
SETAD        = $05
SETSR        = $06
SETWAVE      = $07
SETWAVEPTR   = $08
SETPULSEPTR  = $09
SETFILTPTR   = $0a
SETFILTCTRL  = $0b           ;Resonance/channels
SETFILTCUTOFF = $0c
SETMASTERVOL = $0d
SETFUNKTEMPO = $0e
SETTEMPO     = $0f

        ; Orderlist commands

REPEAT          = $d0
TRANSDOWN       = $e0
TRANS           = $f0
TRANSUP         = $f0
LOOPSONG        = $ff

        ;Wave,pulse,filtertable comands

LOOPWAVE        = $ff
LOOPPULSE       = $ff
LOOPFILTER      = $ff
SETPULSE        = $80
SETFILTER       = $80
SETCUTOFF       = $00

        ;Jump table

music:          jmp mt_init
                jmp mt_play
                jmp mt_stop

        ;Init routine

mt_init:        sta mt_initadc+1
                asl
mt_initadc:     adc #$00
                sta mt_initsongnum+1
                lda #$00
                sta mt_play+1
mt_stopped:     rts

mt_stop:        lda #$ff
                sta mt_play+1
                lda #$00
                sta $d404
                sta $d40b
                sta $d412
                rts

        ;Playroutine

mt_play:        ldx #$00                        ;Channel index
                bne mt_stopped

        ;Song initialization

mt_initsongnum: ldy #$00
                bmi mt_filterstep
                txa
                ldx #41
mt_resetloop:   sta mt_chnsongptr,x             ;Reset sequencer + voice
                dex                             ;variables on all channels
                bpl mt_resetloop
                sta $d415                       ;Reset filter cutoff lowbyte
                sta mt_filterctrl+1             ;Switch filter off & reset
                sta mt_filterstep+1             ;step-programming
                stx mt_initsongnum+1            ;Reset initflag
                tax
                lda startpos
                sta mt_chnsongptr
                lda startpos+1
                sta mt_chnsongptr+7
                lda startpos+2
                sta mt_chnsongptr+14
                jsr mt_initchn
                ldx #$07
                jsr mt_initchn
                ldx #$0e
mt_initchn:     tya
                iny
                sta mt_chnsongnum,x             ;Store index to songtable
                lda mt_chnsongptr,x
                bne mt_nodefaulttempo
                lda instrwavepos+63             ;Is the last instrument unused?
                bne mt_defaulttempo6
                lda instrad+63                  ;If last instrument's AD is nonzero,
                bne mt_defaulttempoad           ;use as the default tempo
mt_defaulttempo6:
                lda #$06                        ;Set default tempo if startpos = 0
mt_defaulttempoad:
                sta mt_chntempo,x
mt_nodefaulttempo:
                lda #$01
                sta mt_chncounter,x             ;Reset counter
                sta mt_chninstr,x               ;Reset instrument
                jmp mt_loadregs2                ;Load waveform

        ;Filter execution

mt_filterstep:  ldy #$00                        ;See if filter stopped
                beq mt_filterdone
mt_filtertime:  lda #$00                        ;See if time left for mod.
                bne mt_filtermod                ;step
mt_newfilterstep:
                lda filttimetbl-1,y             ;$80-> = set filter parameters
                beq mt_setcutoff                ;$00 = set cutoff
                bpl mt_newfiltermod
mt_setfilter:   and #$70                        ;Set passband
                sta mt_filtertype+1
                lda filtspdtbl-1,y              ;Set resonance/channel
                sta mt_filterctrl+1
                lda filttimetbl,y               ;Check for cutoff setting
                bne mt_nextfilterstep2          ;following immediately
mt_setcutoff2:  iny
mt_setcutoff:   lda filtspdtbl-1,y              ;Take cutoff value
                sta mt_filtercutoff+1
                jmp mt_nextfilterstep
mt_newfiltermod:sta mt_filtertime+1             ;$01-$7f = new modulation step
mt_filtermod:   lda filtspdtbl-1,y              ;Take filter speed
                clc
                adc mt_filtercutoff+1
                sta mt_filtercutoff+1
                dec mt_filtertime+1
                bne mt_storecutoff
mt_nextfilterstep:
                lda filttimetbl,y           ;Jump in filtertable?
mt_nextfilterstep2:
                cmp #LOOPFILTER
                iny
                tya
                bcc mt_nofilterjump
                lda filtspdtbl-1,y          ;Take jump point
mt_nofilterjump:sta mt_filterstep+1
mt_filterdone:
mt_filtercutoff:lda #$00
mt_storecutoff: sta $d416
mt_filterctrl:  lda #$00
                sta $d417
mt_filtertype:  lda #$00
mt_masterfader: ora #$0f                        ;Master volume fader
                sta $d418

                jsr mt_execchn
                ldx #$07
                jsr mt_execchn
                ldx #$0e

        ;Channel execution

mt_execchn:     dec mt_chncounter,x               ;See if tick 0
                beq mt_tick0

        ;Ticks 1-n

mt_notick0:     bpl mt_effects
                lda mt_chntempo,x               ;Reload tempo if negative
                cmp #$02
                bcs mt_nofunktempo              ;Funktempo: bounce between
                tay                             ;funktable indexes 0,1
                eor #$01
                sta mt_chntempo,x
                lda mt_funktempotbl,y

mt_nofunktempo: sec
                sbc #$01
                sta mt_chncounter,x
mt_effects:     jmp mt_waveexec

        ;Tick 0

mt_tick0:       ldy mt_chnnewfx,x               ;Setup tick 0 FX jumps
                lda mt_tick0jumptbllo,y
                sta mt_tick0jump1+1
                sta mt_tick0jump2+1
                lda mt_tick0jumptblhi,y
                sta mt_tick0jump1+2
                sta mt_tick0jump2+2

        ;Sequencer advance

mt_checknewpatt:lda mt_chnpattptr,x             ;Fetch next pattern?
                bne mt_nonewpatt
mt_sequencer:   ldy mt_chnsongnum,x
                lda trackadrlo,y                ;Get address of sequence
                sta mt_temp1
                lda trackadrhi,y
                sta mt_temp2
                lda #$10
                sta mt_exitcounter
                ldy mt_chnsongptr,x
mt_seqloop:     lda (mt_temp1),y                ;Get pattern from sequence
                cmp #LOOPSONG                   ;Check for loop
                bcc mt_noloop
                iny
                lda (mt_temp1),y
                tay
mt_seqnext:     dec mt_exitcounter
                bne mt_seqloop                  ;Stop playback if too many commands
                jmp mt_stop

mt_noloop:      cmp #TRANSDOWN                  ;Check for transpose
                bcc mt_notrans
                sbc #TRANS
                sta mt_chntrans,x
                iny
                jmp mt_seqnext

mt_notrans:     cmp #REPEAT                     ;Check for repeat
                bcc mt_norepeat
                sbc #REPEAT
                sta mt_chnrepeat,x
                iny
                jmp mt_seqnext

mt_norepeat:    sta mt_chnpattnum,x             ;Store pattern number
                lda mt_chnrepeat,x
                beq mt_norepeat2
                dec mt_chnrepeat,x
                bpl mt_noadvance
mt_norepeat2:   iny
mt_noadvance:   tya
                sta mt_chnsongptr,x             ;Store songposition

        ;New note start

mt_nonewpatt:   ldy mt_chninstr,x
                lda instrgatetimer,y
                and #$3f
                sta mt_chngatetimer,x
                lda mt_chnnewnote,x             ;Test new note init flag
                beq mt_nonewnoteinit
mt_newnoteinit: sec
                sbc #NOTE
                sta mt_chnnote,x
                lda #$00
                sta mt_chnfx,x                  ;Reset effect
                sta mt_chnnewnote,x             ;Reset newnote action
                lda instrvibdelay,y             ;Load instrument vibrato
                sta mt_chnvibdelay,x
                lda instrvibpos,y
                sta mt_chnparam,x
                lda mt_chnnewfx,x               ;If toneportamento, skip
                cmp #TONEPORTA                  ;most of note init
                beq mt_nonewnoteinit
                lda instrad,y                   ;Load Attack/Decay
                sta mt_chnad,x
                lda instrsr,y
                sta mt_chnsr,x
                lda instrpulsepos,y             ;Load pulseptr (if nonzero)
                beq mt_skippulse
                sta mt_chnpulseptr,x
                lda #$00                        ;Reset pulse step duration
                sta mt_chnpulsetime,x
mt_skippulse:   lda instrfiltpos,y              ;Load filterptr (if nonzero)
                beq mt_skipfilter
                sta mt_filterstep+1
                lda #$00
                sta mt_filtertime+1
mt_skipfilter:
                lda instr1stwave,y              ;Load first frame waveform
                beq mt_skipwave                 ;Skip if "legato instrument"
                cmp #$fe
                bcs mt_skipwave2
                sta mt_chnwave,x
                lda #$ff
mt_skipwave2:   sta mt_chngate,x                ;Reset gateflag
mt_skipwave:    lda instrwavepos,y              ;Load waveptr
                sta mt_chnwaveptr,x
                lda mt_chnnewparam,x            ;Execute tick 0 FX after
mt_tick0jump1:  jsr mt_tick0_0                  ;newnote init
                jmp mt_loadregs

mt_wavecmd:     jmp mt_execwavecmd

        ;Tick 0 effect execution

mt_nonewnoteinit:
                lda mt_chnnewparam,x            ;No new note init: exec tick 0
mt_tick0jump2:  jsr mt_tick0_0                  ;FX, and wavetable afterwards

        ;Wavetable execution

mt_waveexec:    ldy mt_chnwaveptr,x
                beq mt_wavedone
                lda wavetbl-1,y
                cmp #$10                        ;$00-$0f used as delay
                bcs mt_nowavedelay              ;+ no wave change
                cmp mt_chnwavetime,x
                beq mt_nowavechange
                inc mt_chnwavetime,x
                bne mt_wavedone
mt_nowavedelay: cmp #$f0
                bcs mt_nowavechange
                cmp #$e0                       ;$e0-$ef for no waveform
                bcc mt_nowave0f
                and #$0f
mt_nowave0f:    sta mt_chnwave,x
mt_nowavechange:lda wavetbl,y
                cmp #LOOPWAVE                  ;Check for wavetable jump
                iny
                tya
                bcc mt_nowavejump
                clc
                lda notetbl-1,y
mt_nowavejump:  sta mt_chnwaveptr,x
                lda #$00
                sta mt_chnwavetime,x

                lda wavetbl-2,y                 ;Execute command from wavetable
                cmp #$f0
                bcs mt_wavecmd

                lda notetbl-2,y
                cmp #$80
                bne mt_wavefreq                 ;No frequency-change?

        ;No frequency-change / continuous effect execution

mt_wavedone:    lda mt_chncounter,x             ;No continuous effects on tick0
                beq mt_gatetimer
                ldy mt_chnfx,x
                lda mt_effectjumptbllo,y
                sta mt_effectjump+1
                lda mt_effectjumptblhi,y
                sta mt_effectjump+2
                ldy mt_chnparam,x
mt_setspeedparam:
                lda speedlefttbl-1,y
                bmi mt_calculatedspeed
mt_normalspeed: sta mt_temp2
                lda speedrighttbl-1,y
                sta mt_temp1
                jmp mt_effectjump
mt_calculatedspeed:
                lda speedrighttbl-1,y
                sta mt_cscount+1
                sty mt_csresty+1
                ldy mt_chnlastnote,x
                lda mt_freqtbllo+1,y
                sec
                sbc mt_freqtbllo,y
                sta mt_temp1
                lda mt_freqtblhi+1,y
                sbc mt_freqtblhi,y
mt_cscount:     ldy #$00
                beq mt_csresty
mt_csloop:      lsr
                ror mt_temp1
                dey
                bne mt_csloop
mt_csresty:     ldy #$00
                sta mt_temp2
mt_effectjump:  jmp mt_effect_0

        ;Setting note frequency

mt_wavefreq:    bcs mt_wavenoteabs
                adc mt_chnnote,x
mt_wavenoteabs: and #$7f
                sta mt_chnlastnote,x
                tay
mt_wavenote:    lda #$00                        ;Reset vibrato phase
                sta mt_chnvibtime,x

                lda mt_freqtbllo,y
                sta mt_chnfreqlo,x
                lda mt_freqtblhi,y
mt_storefreqhi: sta mt_chnfreqhi,x

        ;Check for new note fetch

mt_done:        lda mt_chncounter,x             ;Check for gateoff timer
mt_gatetimer:
                cmp mt_chngatetimer,x
                beq mt_getnewnote               ;Fetch new notes if equal

        ;Pulse execution

mt_pulseexec:   ldy mt_chnpulseptr,x            ;See if pulse stopped
                beq mt_pulseskip
                lda pulseopt                    ;Pulseoptimization on?
                beq mt_nopulseopt
                lda mt_chncounter,x
                ora mt_chnpattptr,x             ;Skip when sequencer executed
                beq mt_pulseskip
mt_nopulseopt:  lda mt_chnpulsetime,x           ;Pulse step counter time left?
                bne mt_pulsemod
mt_newpulsestep:lda pulsetimetbl-1,y            ;Set pulse, or new modulation
                bpl mt_newpulsemod              ;step?
mt_setpulse:    sta mt_chnpulsehi,x             ;Highbyte
                lda pulsespdtbl-1,y             ;Lowbyte
                sta mt_chnpulselo,x
                jmp mt_nextpulsestep
mt_newpulsemod: sta mt_chnpulsetime,x
mt_pulsemod:    lda pulsespdtbl-1,y          ;Take pulse speed
                clc
                bpl mt_pulseup
                dec mt_chnpulsehi,x
mt_pulseup:     adc mt_chnpulselo,x             ;Add pulse lowbyte
                sta mt_chnpulselo,x
                bcc mt_pulsenotover
                inc mt_chnpulsehi,x
mt_pulsenotover:dec mt_chnpulsetime,x
                bne mt_pulsedone
mt_nextpulsestep:
                lda pulsetimetbl,y           ;Jump in pulsetable?
                cmp #LOOPPULSE
                iny
                tya
                bcc mt_nopulsejump
                lda pulsespdtbl-1,y          ;Take jump point
mt_nopulsejump: sta mt_chnpulseptr,x
mt_pulsedone:
mt_pulseskip:   jmp mt_loadregs

        ;New note fetch

mt_getnewnote:  ldy mt_chnpattnum,x
                lda #$00
                sta mt_temp2
                lda mt_chnpattptr,x
                asl
                rol mt_temp2
                asl
                rol mt_temp2
                adc pattadrlo,y
                sta mt_temp1
                lda mt_temp2
                adc pattadrhi,y
                sta mt_temp2

        ;Check instrument change

                ldy #1
                lda (mt_temp1),y
                beq mt_nonewinstr
                sta mt_chninstr,x

        ;Check effect

mt_nonewinstr:  ldy #2
                lda (mt_temp1),y
                and #$0f
                sta mt_chnnewfx,x
                iny
                lda (mt_temp1),y
                sta mt_chnnewparam,x

        ;Check note

                ldy #0
                lda (mt_temp1),y
mt_note:        cmp #REST                   ;Rest or gateoff/on?
                bcc mt_normalnote
                beq mt_rest
mt_gate:        ora #$f0
                bne mt_setgate

        ;Prepare for note start; perform hardrestart

mt_normalnote:  adc mt_chntrans,x
                sta mt_chnnewnote,x
                lda mt_chnnewfx,x           ;If toneportamento, no gateoff
                cmp #TONEPORTA
                beq mt_rest

                ldy mt_chninstr,x
                lda instrgatetimer,y
                asl                         ;Hardrestart bit to C, legato bit to N
                bmi mt_rest
                bcs mt_skiphr

                lda adparam
                sta mt_chnad,x
                lda srparam
                sta mt_chnsr,x

mt_skiphr:      lda #$fe
mt_setgate:     sta mt_chngate,x

        ;Check for end of pattern

mt_rest:        inc mt_chnpattptr,x
                ldy #4
                lda (mt_temp1),y
                cmp #ENDPATT
                bne mt_loadregs
                lda #$00
                sta mt_chnpattptr,x

        ;Load voice registers

mt_loadregs:    lda mt_chnpulselo,x
                sta $d402,x
                lda mt_chnpulsehi,x
                sta $d403,x    
                lda mt_chnsr,x
                sta $d406,x
                lda mt_chnad,x
                sta $d405,x
                lda mt_chnfreqlo,x
                sta $d400,x
                lda mt_chnfreqhi,x
                sta $d401,x
mt_loadregs2:   lda mt_chnwave,x
                and mt_chngate,x
                sta $d404,x
                rts

        ;0 Instrument vibrato

mt_tick0_0:     lda instrvibpos,y
                jmp mt_tick0_34

        ;1,2 Portamentos

mt_tick0_12:    tay
                lda #$00
                sta mt_chnvibtime,x
                tya

        ;3,4 Toneportamento, Vibrato

mt_tick0_34:    sta mt_chnparam,x
                lda mt_chnnewfx,x
                sta mt_chnfx,x
                rts

        ;5 Set AD

mt_tick0_5:     sta mt_chnad,x
                rts

        ;6 Set Sustain/Release

mt_tick0_6:     sta mt_chnsr,x
                rts

        ;7 Set waveform

mt_tick0_7:     sta mt_chnwave,x
                rts

        ;8 Set wavepointer

mt_tick0_8:     sta mt_chnwaveptr,x
                lda #$00                        ;Make sure possible delayed
                sta mt_chnwavetime,x            ;waveform execution goes
                rts                             ;correctly

        ;9 Set pulsepointer

mt_tick0_9:     sta mt_chnpulseptr,x
                lda #$00                        ;Reset pulse step duration
                sta mt_chnpulsetime,x
                rts

        ;a Set filterpointer

mt_tick0_a:     ldy #$00
                sty mt_filtertime+1
mt_tick0_a_step:sta mt_filterstep+1
                rts

        ;b Set filtercontrol (channels & resonance)

mt_tick0_b:     sta mt_filterctrl+1
                beq mt_tick0_a_step          ;If 0, stop also step-programming
                rts

        ;c Set cutoff

mt_tick0_c:     sta mt_filtercutoff+1
                rts

        ;d Set mastervolume / timing mark

mt_tick0_d:     cmp #$10
                bcs mt_tick0_d_timing
                sta mt_masterfader+1
mt_tick0_d_timing:
                rts

        ;e Funktempo

mt_tick0_e:     tay
                lda speedlefttbl-1,y
                sta mt_funktempotbl
                lda speedrighttbl-1,y
                sta mt_funktempotbl+1
                lda #$00
                beq mt_tick0_f_setglobaltempo

        ;f Set Tempo

mt_tick0_f:     bmi mt_tick0_f_setchantempo     ;Channel or global tempo?
mt_tick0_f_setglobaltempo:
                sta mt_chntempo
                sta mt_chntempo+7
                sta mt_chntempo+14
                rts
mt_tick0_f_setchantempo:
                and #$7f
                sta mt_chntempo,x
                rts

        ;Continuous effect code

        ;0 Instrument vibrato

mt_effect_0:    lda mt_chnvibdelay,x
                beq mt_effect_0_donothing         ;Delay 0 = no vibrato at all
                cmp #$01
                beq mt_effect_4
                dec mt_chnvibdelay,x
mt_effect_0_donothing:
                jmp mt_done

        ;4 Vibrato

mt_effect_4:    lda speedlefttbl-1,y
                bmi mt_effect_4_nohibyteclear
                ldy #$00                          ;Clear speed highbyte
                sty mt_temp2
mt_effect_4_nohibyteclear:
                and #$7f
                sta mt_effect_4_cmp+1
                lda mt_chnvibtime,x
                bmi mt_effect_4_nodir
mt_effect_4_cmp:cmp #$00
                bcc mt_effect_4_nodir2
                beq mt_effect_4_nodir
                eor #$ff
mt_effect_4_nodir:
                clc
mt_effect_4_nodir2:
                adc #$02
mt_vibdone:     sta mt_chnvibtime,x
                lsr
                bcc mt_freqadd
                bcs mt_freqsub

        ;1,2,3 Portamentos

mt_effect_1:    clc
                bcc mt_freqadd
              
mt_effect_2:    sec
                bcs mt_freqsub

mt_effect_3:    tya
                beq mt_effect_3_found           ;Speed $00 = tie note
                sec
                ldy mt_chnnote,x
                lda mt_chnfreqlo,x              ;Calculate offset to the
                                                ;right frequency
                sbc mt_freqtbllo,y
                pha
                lda mt_chnfreqhi,x
                sbc mt_freqtblhi,y

                tay
                pla
                bcs mt_effect_3_down            ;If positive, have to go down

mt_effect_3_up: adc mt_temp1                    ;Add speed to offset
                tya                             ;If changes sign, we're done
                adc mt_temp2
                bpl mt_effect_3_found
mt_freqadd:     lda mt_chnfreqlo,x
                adc mt_temp1
                sta mt_chnfreqlo,x
                lda mt_chnfreqhi,x
                adc mt_temp2
                jmp mt_storefreqhi

mt_effect_3_down:
                sbc mt_temp1                    ;Subtract speed from offset
                tya                             ;If changes sign, we're done
                sbc mt_temp2
                bmi mt_effect_3_found
mt_freqsub:     lda mt_chnfreqlo,x
                sbc mt_temp1
                sta mt_chnfreqlo,x
                lda mt_chnfreqhi,x
                sbc mt_temp2
                jmp mt_storefreqhi

mt_effect_3_found:
                ldy mt_chnnote,x
                jmp mt_wavenote

mt_execwavecmd: and #$0f
                sta mt_temp1
                lda notetbl-2,y
                sta mt_temp2
                ldy mt_temp1
                cpy #$05
                bcs mt_execwavetick0
mt_execwavetickn:
                lda mt_effectjumptbllo,y
                sta mt_effectjump+1
                lda mt_effectjumptblhi,y
                sta mt_effectjump+2
                ldy mt_temp2
                jmp mt_setspeedparam
mt_execwavetick0:
                lda mt_tick0jumptbllo,y
                sta mt_execwavetick0jump+1
                lda mt_tick0jumptblhi,y
                sta mt_execwavetick0jump+2
                lda mt_temp2
mt_execwavetick0jump:
                jsr mt_tick0_0
                jmp mt_done

mt_chnsongptr:    dc.b 0
mt_chntrans:      dc.b 0
mt_chnrepeat:     dc.b 0
mt_chnpattptr:    dc.b 0
mt_chnpackedrest: dc.b 0
mt_chnnewfx:      dc.b 0
mt_chnnewparam:   dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

mt_chnfx:         dc.b 0
mt_chnparam:      dc.b 0
mt_chnnewnote:    dc.b 0
mt_chnwaveptr:    dc.b 0
mt_chnwave:       dc.b 0
mt_chnpulseptr:   dc.b 0
mt_chnpulsetime:  dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

mt_chnsongnum:    dc.b 0
mt_chnpattnum:    dc.b 0
mt_chnnote:       dc.b 0
mt_chnfreqlo:     dc.b 0
mt_chnfreqhi:     dc.b 0
mt_chnpulselo:    dc.b 0
mt_chnpulsehi:    dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

mt_chninstr:      dc.b 1
mt_chntempo:      dc.b 0
mt_chncounter:    dc.b 0
mt_chnvibtime:    dc.b 0
mt_chnvibdelay:   dc.b 0
mt_chnwavetime:   dc.b 0
mt_chngate:       dc.b $fe

                dc.b 1,0,0,0,0,0,$fe
                dc.b 1,0,0,0,0,0,$fe
                
mt_chnad:         dc.b 0
mt_chnsr:         dc.b 0
mt_chngatetimer:  dc.b 0
mt_chnlastnote:   dc.b 0
mt_unused3:       dc.b 0
mt_unused4:       dc.b 0
mt_unused5:       dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0


mt_freqtblhi:   dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02
                dc.b $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04
                dc.b $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08
                dc.b $08,$09,$09,$0a,$0a,$0b,$0c,$0d,$0d,$0e,$0f,$10
                dc.b $11,$12,$13,$14,$15,$17,$18,$1a,$1b,$1d,$1f,$20
                dc.b $22,$24,$27,$29,$2b,$2e,$31,$34,$37,$3a,$3e,$41
                dc.b $45,$49,$4e,$52,$57,$5c,$62,$68,$6e,$75,$7c,$83
                dc.b $8b,$93,$9c,$a5,$af,$b9,$c4,$d0,$dd,$ea,$f8,$ff

mt_freqtbllo:   dc.b $17,$27,$39,$4b,$5f,$74,$8a,$a1,$ba,$d4,$f0,$0e
                dc.b $2d,$4e,$71,$96,$be,$e8,$14,$43,$74,$a9,$e1,$1c
                dc.b $5a,$9c,$e2,$2d,$7c,$cf,$28,$85,$e8,$52,$c1,$37
                dc.b $b4,$39,$c5,$5a,$f7,$9e,$4f,$0a,$d1,$a3,$82,$6e
                dc.b $68,$71,$8a,$b3,$ee,$3c,$9e,$15,$a2,$46,$04,$dc
                dc.b $d0,$e2,$14,$67,$dd,$79,$3c,$29,$44,$8d,$08,$b8
                dc.b $a1,$c5,$28,$cd,$ba,$f1,$78,$53,$87,$1a,$10,$71
                dc.b $42,$89,$4f,$9b,$74,$e2,$f0,$a6,$0e,$33,$20,$ff

mt_effectjumptbllo:
                dc.b <mt_effect_0
                dc.b <mt_effect_1
                dc.b <mt_effect_2
                dc.b <mt_effect_3
                dc.b <mt_effect_4

mt_effectjumptblhi:
                dc.b >mt_effect_0
                dc.b >mt_effect_1
                dc.b >mt_effect_2
                dc.b >mt_effect_3
                dc.b >mt_effect_4

mt_tick0jumptbllo:
                dc.b <mt_tick0_0
                dc.b <mt_tick0_12
                dc.b <mt_tick0_12
                dc.b <mt_tick0_34
                dc.b <mt_tick0_34
                dc.b <mt_tick0_5
                dc.b <mt_tick0_6
                dc.b <mt_tick0_7
                dc.b <mt_tick0_8
                dc.b <mt_tick0_9
                dc.b <mt_tick0_a
                dc.b <mt_tick0_b
                dc.b <mt_tick0_c
                dc.b <mt_tick0_d
                dc.b <mt_tick0_e
                dc.b <mt_tick0_f

mt_tick0jumptblhi:
                dc.b >mt_tick0_0
                dc.b >mt_tick0_12
                dc.b >mt_tick0_12
                dc.b >mt_tick0_34
                dc.b >mt_tick0_34
                dc.b >mt_tick0_5
                dc.b >mt_tick0_6
                dc.b >mt_tick0_7
                dc.b >mt_tick0_8
                dc.b >mt_tick0_9
                dc.b >mt_tick0_a
                dc.b >mt_tick0_b
                dc.b >mt_tick0_c
                dc.b >mt_tick0_d
                dc.b >mt_tick0_e
                dc.b >mt_tick0_f

mt_funktempotbl:dc.b 9,6

mt_exitcounter: dc.b 0

freemem:
